<?php

namespace App\Models;

use EscapeWork\LaraMedias\Contracts\Mediable;
use EscapeWork\LaraMedias\Traits\Medias;
use EscapeWork\LaravelSteroids\Cacheable;
use EscapeWork\LaravelSteroids\Model;
use EscapeWork\LaravelSteroids\Presentable;
use EscapeWork\LaravelSteroids\Sluggable;
use Illuminate\Support\Facades\DB;

class Product extends Model implements Mediable
{
    /**
     * Table
     */
    protected $table = 'products';

    /**
     * Fillable fields
     */
    protected $fillable = [
        'collection_id',
        'category_id',
        'subcategory_id',
        'title',
        'slug',
        'reference',
        'description',
        'color',
        'buy_link',
        'active',
        'featured',
        'functional',
    ];

    /**
     * Traits
     */
    use Presentable, Sluggable, Cacheable, Medias;

    /**
     * Presenter class
     */
    protected $presenter = 'App\Presenters\ProductPresenter';

    /**
     * Cacheables
     */
    protected $cacheables = [
        'products.featured',
    ];

    public function collection()
    {
        return $this->belongsTo('App\Models\Collection', 'collection_id');
    }

    public function category()
    {
        return $this->belongsTo('App\Models\Category', 'category_id');
    }

    public function subcategory()
    {
        return $this->belongsTo('App\Models\Category', 'subcategory_id');
    }

    public function colors()
    {
        return $this->hasMany('App\Models\Product', 'reference', 'reference')
                    ->where('id', '<>', $this->id);
    }

    public function related()
    {
        return $this->hasMany('App\Models\Product', 'subcategory_id', 'subcategory_id')
                    ->where('id', '<>', $this->id)
                    ->where('reference', '<>', $this->reference)
                    ->groupBy('reference')
                    ->take(3);
    }

    public static function sync($subcategory, $data)
    {
        $product = static::where('title', '=', $data['title'])
                         ->where('description', '=', $data['description'])
                         ->where('collection_id', '=', $subcategory->collection_id)
                         ->where('category_id', '=', $subcategory->category_id)
                         ->where('subcategory_id', '=', $subcategory->id)
                         ->first();

        if (! $product) {
            $product = static::create(array_merge([
                'collection_id'  => $subcategory->collection_id,
                'category_id'    => $subcategory->category_id,
                'subcategory_id' => $subcategory->id,
            ], $data));
        } else {
            $product->fill($data);
            $product->save();
        }

        return $product;
    }

    public function medias()
    {
        return $this->morphMany('EscapeWork\LaraMedias\Models\Media', 'medias', 'media_model', 'model_id')
                    ->orderBy('order');
    }

    public function scopeFeatured($query)
    {
        return $query->where('featured', '=', 1);
    }

    public function getRouteKeyName()
    {
        return 'slug';
    }

    public function uploadLogo($card)
    {
        if ($card) {
            $this->uploadSingleMedia($card, 'card');
            $this->save();
        }
    }

    public function scopeJoinCategories($query)
    {
        return $query->join('categories', 'categories.id', '=', 'products.category_id');
    }

    public function scopeJoinSubcategories($query)
    {
        return $query->join(DB::raw('categories subcategories'), 'subcategories.id', '=', 'products.subcategory_id');
    }

    public function scopeActive($query)
    {
        return $query->where('products.active', '=', 1);
    }

    public function scopeRandom($query)
    {
        return $query->orderByRaw('RAND()');
    }

    public function scopeFunctional($query)
    {
        return $query->where('functional', '=', 1);
    }
}
