<?php

namespace EscapeWork\Manager\Console;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Str;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;

class CrudGeneratorCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'manager:crud {--model=} {--title=} {--schema=}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cria um CRUD base (Controller, Requests, Model, Repository e Commands)';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function handle()
    {
        if (! $model = $this->option('model')) {
            $model = Str::title($this->ask('Qual o nome do model? (ex.: Product, Service, News)'));
        }

        $title  = $this->option('title');
        $names  = $this->getNames($model);
        $medias = $this->confirm('Este model vai utilizar mídias?');

        if ($this->confirm('O nome da variável no singular vai ser $'.$names['variable'].'. Deseja alterar?')) {
            $names['variable'] = $this->ask('Digite o nome da variável no SINGULAR', '$' . $names['variable']);

            if (strpos($names['variable'], '$') === false) {
                $names['variable'] = '$' . $names['variable'];
            }
        }

        if (is_null($title)) {
            $title = $this->ask('Qual o título da página (ex.: Produtos, Serviços, Categorias)?');
        }

        $singular = Str::lower($this->ask('Qual o singular do nome da página (ex.: Produto, Serviço, Categoria)?'));

        # criando a migration
        $schema = $this->option('schema');
        if (is_null($schema) and $this->confirm('Deseja criar uma migration junto?')) {
            $this->call('make:migration', [
                'name'     => 'create_' . $names['table'] . '_table',
                '--create' => $names['table']
            ]);
        }

        # creating views where user will be asked for inputs if hasnt provided
        $viewOptions = [
            'title'      => $title,
            '--singular' => $singular,
            '--variable' => $names['variable'],
        ];

        if (! is_null($schema)) {
            $viewOptions['--schema'] = $schema;
        }

        $this->call('manager:make:views', $viewOptions);

        # criando o repo
        $this->call('manager:make:repository', [
            'model' => $model,
        ]);

        # criando o model
        $this->call('manager:make:model', [
            'model'    => $model,
            '--medias' => $medias,
        ]);

        # criando os requests
        $this->call('manager:make:request', [
            'model' => $model,
        ]);

        # criando o controller
        $this->call('manager:make:controller', [
            'model'      => $model,
            '--singular' => $singular,
        ]);

        $this->line('');
        $this->info(' -> Seu CRUD foi gerado com sucesso!');

        $this->line('');
        $this->comment(' -> Registre as rotas abaixo no seu arquivo de rotas.');
        $this->info(' -> IMPORTANTE: Coloque estas rotas dentro de um grupo com o prefixo da URL do manager e validaçnao de autenticação!!!');

        $route      = Str::slug(Str::plural($title));
        $routeAlias = Str::slug(Str::plural($model));
        $routes     = '';

        $routes .= "Route::get('".$route."', ['as' => '".$routeAlias.".index', 'uses' => '".$names['controller']."@index']);\n";
        $routes .= "Route::get('".$route."/inserir', ['as' => '".$routeAlias.".create', 'uses' => '".$names['controller']."@create']);\n";
        $routes .= "Route::post('".$route."', ['as' => '".$routeAlias.".store', 'uses' => '".$names['controller']."@store']);\n";
        $routes .= "Route::get('".$route."/{id}/editar', ['as' => '".$routeAlias.".edit', 'uses' => '".$names['controller']."@edit']);\n";
        $routes .= "Route::put('".$route."/{id}', ['as' => '".$routeAlias.".update', 'uses' => '".$names['controller']."@update']);\n";
        $routes .= "Route::put('".$route."/{id}/basic', ['as' => '".$routeAlias.".updateBasic', 'uses' => '".$names['controller']."@updateBasic']);\n";
        $routes .= "Route::delete('".$route."/{id}', ['as' => '".$routeAlias.".destroy', 'uses' => '".$names['controller']."@destroy']);";

        # adicionando as rotas no arquivo
        File::append(base_path('routes/manager.php'), $routes);

        # adicionando no menu
        $html = '<li class="{{ $page == \''.$routeAlias.'\' ? \'active\' : null }}">
    <a href="{{ route(\'manager.'.$routeAlias.'.index\') }}">' . $title . '</a>
</li>';

        if (\File::exists($menu = base_path().'/resources/views/manager/menu.blade.php')) {
            \File::append($menu, "\n" . $html);

            $this->line('');
            $this->info(' -> Adicionamos o HTML com o link na view do menu (resources/views/manager/menu.blade)');
        } else {
            $this->line('');
            $this->comment(' -> E adicione no seu menu');
            $this->line($html);
        }
    }

    protected function getNames($name)
    {
        $plural = Str::plural($name);

        return [
            'variable'   => Str::lower($name),
            'table'      => Str::lower($plural),
            'controller' => $plural . 'Controller',
            'model'      => $name,
            'repository' => $name . 'Repository',
        ];
    }
}
