<?php

namespace EscapeWork\Manager\Console;

use Illuminate\Support\Str;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Illuminate\Container\Container;

class CrudGeneratorTemplateCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'manager:crud:template {template?} {--with-migration}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Cria um CRUD com base em um template pré definido (Opções: news)';

    /**
     * @var  mixed
     */
    protected $key, $namespace, $templates, $template;

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function handle()
    {
        $this->templates = require(__DIR__ . '/../../config/templates.php');

        if (! $this->argument('template') || ! array_key_exists($this->argument('template'), $this->templates )) {
            $this->showOptions($this->templates);
            return;
        }

        // se houver dependencias, instala antes do modelo
        if( isset($this->templates[$this->argument('template')]['dependencies']) ){
          foreach( (array) $this->templates[$this->argument('template')]['dependencies'] as $dependency_key){
              $this->template = $dependency_key;
              $this->generate($dependency_key);
          }

        }

        $this->key = $this->argument('template');
        $this->template = $this->key;
        $this->generate($this->key );

        // se houver complementos, instala depois do modelo
        if( isset($this->templates[$this->argument('template')]['complements']) ){
          foreach( (array) $this->templates[$this->argument('template')]['complements'] as $complement_key){
              $this->template = $complement_key;
              $this->generate($complement_key);
          }
        }

    }

    public function generate( $key){
        $template        = $this->templates[$key];
        $this->namespace = Container::getInstance()->getNamespace();

        $this->info(' -> Gerando os arquivos para o ' . $template['description']);
        $this->line('');

        foreach ($template['files'] as $file => $data) {
            $path = base_path() . '/' . $data['path'];

            # se for pasta, copia o diretório inteiro
            if ($data['extension'] == 'folder') {
                $this->copyFolder($path, $data);
                continue;
            }

            # se não, cria os arquivos
            $this->createFile($path, isset($data['name']) ? $data['name'] : $file, $data);
        }

        $this->line('');
        $this->info(' -> Cadastro criado com sucesso!');
        $this->line('');

        # rotas
        $this->appendRoutes($template);

        # menu
        $this->addToMenu($template);

        if ($this->option('with-migration')) {
            $this->info(' -> E não esqueça de executar a migration!');
        }

        if (isset($template['obs']) && $template['obs'] !== '') {
            $this->comment(' -> ' . $template['obs']);
        }
    }

    public function copyFolder($path, $data)
    {
        File::copyDirectory(
            __DIR__ . '/../../resources/stubs/'.$this->template.'/' . $data['stub'], # diretório original
            $path
        );

        $this->comment(' -> Diretório ' . $data['path'] . ' criado!');
    }

    public function createFile($path, $file, $data)
    {
        $stub = File::get(__DIR__ . '/../../resources/stubs/'.$this->template.'/' . $data['stub']);
        $stub = str_replace('{{namespace}}', $this->namespace, $stub);
        $file = str_replace('{{datetime}}', date('Y_m_d_His'), $file);

        # criando o diretório se não existe
        if (! File::isDirectory($path)) {
            File::makeDirectory($path);
        }

        if (! File::exists($path . '/' . $file . $data['extension'])) {
            File::put($path . '/' . $file . $data['extension'], $stub);

            $this->comment(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' criado com sucesso!');
        } else {
            $this->error(' -> ' . $data['path'] . '/' . $file . $data['extension'] . ' já existe e não foi sobreescrito.');
        }
    }

    public function showOptions($templates)
    {
        $this->info('Os templates disponíveis são os seguintes: ');
        $this->line('');

        $headers = ['Template', 'Descrição', 'Uso'];
        $content = [];

        foreach ($templates as $type => $template) {
            $content[] = [
                $type, $template['description'], 'php artisan manager:crud:template ' . $type
            ];
        }

        $this->table($headers, $content);
    }

    protected function appendRoutes($template)
    {
        // -------------------------------------------------------
        // rotas do manager
        $routes = "\n";

        if (is_array($template['route'])) {
            $routes .= "Route::get('".$template['route']['url']."', '".$template['route']['controller']."@index')->name('".$template['route']['name'].".index'); \n";
            $routes .= "Route::get('".$template['route']['url']."/inserir', '".$template['route']['controller']."@create')->name('".$template['route']['name'].".create'); \n";
            $routes .= "Route::post('".$template['route']['url']."', '".$template['route']['controller']."@store')->name('".$template['route']['name'].".store'); \n";
            $routes .= "Route::get('".$template['route']['url']."/{id}/editar', '".$template['route']['controller']."@edit')->name('".$template['route']['name'].".edit'); \n";
            $routes .= "Route::put('".$template['route']['url']."/{id}', '".$template['route']['controller']."@update')->name('".$template['route']['name'].".update'); \n";
            $routes .= "Route::put('".$template['route']['url']."/{id}/basic', '".$template['route']['controller']."@updateBasic')->name('".$template['route']['name'].".updateBasic'); \n";
            $routes .= "Route::delete('".$template['route']['url']."/{id}', '".$template['route']['controller']."@destroy')->name('".$template['route']['name'].".destroy'); \n";
        } else {
            $routes = File::get(__DIR__ . '/../../resources/stubs/'.$this->template.'/' . $template['route']);
        }

        # adicionando as rotas no arquivo
        File::append(base_path('routes/manager.php'), $routes);
        $this->info(' -> Rotas do manager adicionadas com sucesso!');

        // -------------------------------------------------------
        // rotas do app (se existir)
        if (isset($template['routes-app'])) {
            File::append(base_path('routes/web.php'), File::get(__DIR__ . '/../../resources/stubs/'.$this->template.'/' . $template['routes-app']));
            $this->info(' -> Rotas do app adicionadas com sucesso!');
        }
    }

    protected function addToMenu($template)
    {
        if (! is_array($template['route']) && ! isset($template['menu'])) {
            return;
        }

        # adicionando no menu
        if (! isset($template['menu'])) {
            $html = '<li class="{{ $page == \''.$template['route']['name'].'\' ? \'active\' : null }}">
        <a href="{{ route(\'manager.'.$template['route']['name'].'.index\') }}">' . $template['title'] . '</a>
    </li>';
        } else {
            $html = File::get(__DIR__ . '/../../resources/stubs/'.$this->template.'/' . $template['menu']);
        }

        if (File::exists($menu = base_path('resources/views/manager/menu.blade.php'))) {
            File::append($menu, "\n" . $html);

            $this->line('');
            $this->info(' -> Adicionamos o HTML com o link na view do menu (resources/views/manager/menu.blade.php)');
        } else {
            $this->line('');
            $this->comment(' -> E adicione no seu menu');
            $this->line($html);
        }
    }
}
