<?php

namespace EscapeWork\Manager\Console;

use Illuminate\Console\Command;
use Illuminate\Filesystem\Filesystem;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Illuminate\Support\Str;
use Illuminate\Container\Container;

abstract class GeneratorCommand extends Command
{
    /**
     * The filesystem instance.
     *
     * @var \Illuminate\Filesystem\Filesystem
     */
    protected $files;

    /**
     * The type of class being generated.
     *
     * @var string
     */
    protected $type;

    /**
     * Create a new controller creator command instance.
     *
     * @param  \Illuminate\Filesystem\Filesystem  $files
     * @return void
     */
    public function __construct(Filesystem $files)
    {
        parent::__construct();

        $this->files = $files;
    }

    /**
     * Get the stub file for the generator.
     *
     * @return string
     */
    abstract protected function getStub();

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function handle()
    {
        $model = $this->parseName($this->getNameInput());
        $model .= $this->type == 'Model' ? null : $this->type;

        if ($this->files->exists($path = $this->getPath($model)))
        {
            return $this->error($this->type.' already exists!');
        }

        $this->makeDirectory($path);

        $this->files->put($path, $this->buildClass($model));

        $this->info($this->type.' created successfully.');
    }

    /**
     * Get the destination class path.
     *
     * @param  string  $name
     * @return string
     */
    protected function getPath($name)
    {
        $name = str_replace($this->getAppNamespace(), '', $name);

        return $this->laravel['path'].'/'.str_replace('\\', '/', $name).'.php';
    }

    /**
     * Parse the name and format according to the root namespace.
     *
     * @param  string  $name
     * @return string
     */
    protected function parseName($name)
    {
        $rootNamespace = $this->getAppNamespace();

        if (starts_with($name, $rootNamespace))
        {
            return $name;
        }

        if (str_contains($name, '/'))
        {
            $name = str_replace('/', '\\', $name);
        }

        return $this->parseName($this->getDefaultNamespace(trim($rootNamespace, '\\')).'\\'.$name);
    }

    /**
     * Get the default namespace for the class.
     *
     * @param  string  $rootNamespace
     * @return string
     */
    protected function getDefaultNamespace($rootNamespace)
    {
        return $rootNamespace;
    }

    /**
     * Build the directory for the class if necessary.
     *
     * @param  string  $path
     * @return string
     */
    protected function makeDirectory($path)
    {
        if ( ! $this->files->isDirectory(dirname($path)))
        {
            $this->files->makeDirectory(dirname($path), 0777, true, true);
        }
    }

    /**
     * Build the class with the given name.
     *
     * @param  string  $name
     * @return string
     */
    protected function buildClass($model)
    {
        $stub = $this->files->get($this->getStub());
        $stub = $this->replaceNamespace($stub, $model);
        $stub = $this->replaceVariables($stub, $model);
        $stub = $this->replaceClass($stub, $model);
        $stub = $this->replaceSpecific($stub, $model);

        return $stub;
    }

    public function replaceSpecific($stub, $name)
    {
        return $stub;
    }

    protected function replaceVariables($stub, $name)
    {
        $variable = Str::lower($this->argument('model'));
        $stub = str_replace('{{variable}}', $variable, $stub);

        return $stub;
    }

    protected function replaceNamespace(&$stub, $name)
    {
        $stub = str_replace(
            '{{namespace}}', $this->getNamespace($name), $stub
        );

        $stub = str_replace(
            '{{rootNamespace}}', $this->getAppNamespace(), $stub
        );

        return $stub;
    }

    protected function getNamespace($model)
    {
        return trim(implode('\\', array_slice(explode('\\', $model), 0, -1)), '\\');
    }

    /**
     * Replace the class name for the given stub.
     *
     * @param  string  $stub
     * @param  string  $name
     * @return string
     */
    protected function replaceClass($stub, $model)
    {
        $class = str_replace($this->getNamespace($model).'\\', '', $model);

        return str_replace('{{class}}', $class, $stub);
    }

    /**
     * Get the desired class name from the input.
     *
     * @return string
     */
    protected function getNameInput($plural = false)
    {
        if ($this->type == 'Controller') {
            return Str::plural($this->argument('model'));
        }

        return $this->argument('model');
    }

    /**
     * Get the appnamespace
     *
     * @return string
     */
    protected function getAppNamespace()
    {
        return Container::getInstance()->getNamespace();
    }

}
